@@ -39,6 +39,7 @@ class SimpleEnv(AECEnv):
         # set spaces
         self.action_spaces = dict()
         self.observation_spaces = dict()
+        state_dim = 0
         for agent in self.world.agents:
             space_dim = 1
             if agent.movable:
@@ -47,9 +48,12 @@ class SimpleEnv(AECEnv):
                 space_dim *= self.world.dim_c
 
             obs_dim = len(self.scenario.observation(agent, self.world))
+            state_dim += obs_dim
             self.action_spaces[agent.name] = spaces.Discrete(space_dim)
             self.observation_spaces[agent.name] = spaces.Box(low=-np.float32(np.inf), high=+np.float32(np.inf), shape=(obs_dim,), dtype=np.float32)
 
+        self.state_space = spaces.Box(low=-np.float32(np.inf), high=+np.float32(np.inf), shape=(state_dim,), dtype=np.float32)
+
         self.steps = 0
 
         self.current_actions = [None] * self.num_agents
@@ -62,6 +66,10 @@ class SimpleEnv(AECEnv):
     def observe(self, agent):
         return self.scenario.observation(self.world.agents[self._index_map[agent]], self.world).astype(np.float32)
 
+    def state(self):
+        states = tuple([self.scenario.observation(self.world.agents[self._index_map[agent]], self.world).astype(np.float32) for agent in self.possible_agents])
+        return np.concatenate(states, axis=None)
+
     def reset(self):
         self.scenario.reset_world(self.world, self.np_random)
 
@@ -161,7 +169,6 @@ class SimpleEnv(AECEnv):
 
         self._cumulative_rewards[cur_agent] = 0
         self._accumulate_rewards()
-        self._dones_step_first()
 
     def render(self, mode='human'):
         from . import rendering
