@@ -3,7 +3,7 @@ from pettingzoo import AECEnv
 from pettingzoo.utils import agent_selector
 from pettingzoo.utils import wrappers
 import numpy as np
-from pettingzoo.utils.to_parallel import parallel_wrapper_fn
+from pettingzoo.utils.conversions import parallel_wrapper_fn
 
 
 def env(**kwargs):
@@ -32,7 +32,6 @@ class raw_env(AECEnv):
         self.action_spaces = dict(zip(self.agents, self.env.action_space))
         self.observation_spaces = dict(
             zip(self.agents, self.env.observation_space))
-        self.display_wait = 0.03
         self.has_reset = False
 
     def seed(self, seed=None):
@@ -80,7 +79,6 @@ class raw_env(AECEnv):
         self._cumulative_rewards[self.agent_selection] = 0
         self.agent_selection = self._agent_selector.next()
         self._accumulate_rewards()
-        self._dones_step_first()
 
     def observe(self, agent):
         return self.env.observe(self.agent_name_mapping[agent])
